@@ -30,6 +30,7 @@ typedef enum {
   CLOG_SEC = 2,
   CLOG_WARN = 3,
   CLOG_ERROR = 4,
+  CLOG_UNKNOWN = -1,
 } clog_type;
 
 static const std::string CLOG_CHANNEL_NONE    = "none";
@@ -42,6 +43,7 @@ static const std::string CLOG_CHANNEL_AUDIT   = "audit";
  */
 int clog_type_to_syslog_level(clog_type t);
 
+clog_type string_to_clog_type(const string& s);
 int string_to_syslog_level(string s);
 int string_to_syslog_facility(string s);
 
@@ -75,7 +77,7 @@ struct LogEntry {
   string msg;
   string channel;
 
-  LogEntry() : seq(0) {}
+  LogEntry() : seq(0), prio(CLOG_DEBUG) {}
 
   LogEntryKey key() const { return LogEntryKey(who, stamp, seq); }
 
@@ -122,12 +124,12 @@ inline ostream& operator<<(ostream& out, clog_type t)
     return out << "[DBG]";
   case CLOG_INFO:
     return out << "[INF]";
+  case CLOG_SEC:
+    return out << "[SEC]";
   case CLOG_WARN:
     return out << "[WRN]";
   case CLOG_ERROR:
     return out << "[ERR]";
-  case CLOG_SEC:
-    return out << "[SEC]";
   default:
     return out << "[???]";
   }
