@@ -72,13 +72,7 @@ namespace librbd {
     ~AioCompletion() {
     }
 
-    int wait_for_complete() {
-      lock.Lock();
-      while (!done)
-	cond.Wait(lock);
-      lock.Unlock();
-      return 0;
-    }
+    int wait_for_complete();
 
     void add_request() {
       lock.Lock();
@@ -93,16 +87,11 @@ namespace librbd {
 
     void init_time(ImageCtx *i, aio_type_t t) {
       ictx = i;
-      {
-        Mutex::Locker l(ictx->aio_lock);
-        ++ictx->pending_aio;
-      }
       aio_type = t;
       start_time = ceph_clock_now(ictx->cct);
     }
 
-    void complete(CephContext *cct);
-    void fail(CephContext *cct, int r);
+    void complete();
 
     void set_complete_cb(void *cb_arg, callback_t cb) {
       complete_cb = cb;
@@ -111,17 +100,9 @@ namespace librbd {
 
     void complete_request(CephContext *cct, ssize_t r);
 
-    bool is_complete() {
-      Mutex::Locker l(lock);
-      return done;
-    }
+    bool is_complete();
 
-    ssize_t get_return_value() {
-      lock.Lock();
-      ssize_t r = rval;
-      lock.Unlock();
-      return r;
-    }
+    ssize_t get_return_value();
 
     void get() {
       lock.Lock();
