@@ -21,10 +21,13 @@ class Log : private Thread
 
   SubsystemMap *m_subs;
   
-  pthread_spinlock_t m_lock;
   pthread_mutex_t m_queue_mutex;
   pthread_mutex_t m_flush_mutex;
-  pthread_cond_t m_cond;
+  pthread_cond_t m_cond_loggers;
+  pthread_cond_t m_cond_flusher;
+
+  pthread_t m_queue_mutex_holder;
+  pthread_t m_flush_mutex_holder;
 
   EntryQueue m_new;    ///< new entries
   EntryQueue m_recent; ///< recent (less new) entries we've already written at low detail
@@ -39,6 +42,8 @@ class Log : private Thread
 
   int m_max_new, m_max_recent;
 
+  bool m_inject_segv;
+
   void *entry();
 
   void _flush(EntryQueue *q, EntryQueue *requeue, bool crash);
@@ -49,6 +54,8 @@ public:
   Log(SubsystemMap *s);
   virtual ~Log();
 
+  void set_flush_on_exit();
+
   void set_max_new(int n);
   void set_max_recent(int n);
   void set_log_file(std::string fn);
@@ -66,6 +73,12 @@ public:
 
   void start();
   void stop();
+
+  /// true if the log lock is held by our thread
+  bool is_inside_log_lock();
+
+  /// induce a segv on the next log event
+  void inject_segv();
 };
 
 }
