@@ -12,17 +12,29 @@
 #include "include/assert.h"
 #include "osd/osd_types.h"
 
+#ifdef WITH_LTTNG
+#include "tracing/oprequest.h"
+#endif
 
 
 OpRequest::OpRequest(Message *req, OpTracker *tracker) :
-  TrackedOp(req, tracker),
-  rmw_flags(0),
+  TrackedOp(tracker, req->get_recv_stamp()),
+  rmw_flags(0), request(req),
   hit_flag_points(0), latest_flag_point(0),
   send_map_update(false), sent_epoch(0) {
   if (req->get_priority() < tracker->cct->_conf->osd_client_op_priority) {
     // don't warn as quickly for low priority ops
     warn_interval_multiplier = tracker->cct->_conf->osd_recovery_op_warn_multiple;
   }
+  if (req->get_type() == CEPH_MSG_OSD_OP) {
+    reqid = static_cast<MOSDOp*>(req)->get_reqid();
+  } else if (req->get_type() == MSG_OSD_SUBOP) {
+    reqid = static_cast<MOSDSubOp*>(req)->reqid;
+  }
+  tracker->mark_event(this, "header_read", request->get_recv_stamp());
+  tracker->mark_event(this, "throttled", request->get_throttle_stamp());
+  tracker->mark_event(this, "all_read", request->get_recv_complete_stamp());
+  tracker->mark_event(this, "dispatched", request->get_dispatch_stamp());
 }
 
 void OpRequest::_dump(utime_t now, Formatter *f) const
@@ -34,11 +46,12 @@ void OpRequest::_dump(utime_t now, Formatter *f) const
     stringstream client_name;
     client_name << m->get_orig_source();
     f->dump_string("client", client_name.str());
-    f->dump_int("tid", m->get_tid());
+    f->dump_unsigned("tid", m->get_tid());
     f->close_section(); // client_info
   }
   {
     f->open_array_section("events");
+    Mutex::Locker l(lock);
     for (list<pair<utime_t, string> >::const_iterator i = events.begin();
 	 i != events.end();
 	 ++i) {
@@ -51,13 +64,14 @@ void OpRequest::_dump(utime_t now, Formatter *f) const
   }
 }
 
-void OpRequest::init_from_message()
+void OpRequest::_dump_op_descriptor_unlocked(ostream& stream) const
 {
-  if (request->get_type() == CEPH_MSG_OSD_OP) {
-    reqid = static_cast<MOSDOp*>(request)->get_reqid();
-  } else if (request->get_type() == MSG_OSD_SUBOP) {
-    reqid = static_cast<MOSDSubOp*>(request)->reqid;
-  }
+  get_req()->print(stream);
+}
+
+void OpRequest::_unregistered() {
+  request->clear_data();
+  request->clear_payload();
 }
 
 bool OpRequest::check_rmw(int flag) {
@@ -79,9 +93,33 @@ bool OpRequest::need_class_read_cap() {
 bool OpRequest::need_class_write_cap() {
   return check_rmw(CEPH_OSD_RMW_FLAG_CLASS_WRITE);
 }
-void OpRequest::set_read() { rmw_flags |= CEPH_OSD_RMW_FLAG_READ; }
-void OpRequest::set_write() { rmw_flags |= CEPH_OSD_RMW_FLAG_WRITE; }
-void OpRequest::set_class_read() { rmw_flags |= CEPH_OSD_RMW_FLAG_CLASS_READ; }
-void OpRequest::set_class_write() { rmw_flags |= CEPH_OSD_RMW_FLAG_CLASS_WRITE; }
-void OpRequest::set_pg_op() { rmw_flags |= CEPH_OSD_RMW_FLAG_PGOP; }
-void OpRequest::set_cache() { rmw_flags |= CEPH_OSD_RMW_FLAG_CACHE; }
+
+void OpRequest::set_rmw_flags(int flags) {
+#ifdef WITH_LTTNG
+  int old_rmw_flags = rmw_flags;
+#endif
+  rmw_flags |= flags;
+  tracepoint(oprequest, set_rmw_flags, reqid.name._type,
+	     reqid.name._num, reqid.tid, reqid.inc,
+	     flags, old_rmw_flags, rmw_flags);
+}
+
+void OpRequest::set_read() { set_rmw_flags(CEPH_OSD_RMW_FLAG_READ); }
+void OpRequest::set_write() { set_rmw_flags(CEPH_OSD_RMW_FLAG_WRITE); }
+void OpRequest::set_class_read() { set_rmw_flags(CEPH_OSD_RMW_FLAG_CLASS_READ); }
+void OpRequest::set_class_write() { set_rmw_flags(CEPH_OSD_RMW_FLAG_CLASS_WRITE); }
+void OpRequest::set_pg_op() { set_rmw_flags(CEPH_OSD_RMW_FLAG_PGOP); }
+void OpRequest::set_cache() { set_rmw_flags(CEPH_OSD_RMW_FLAG_CACHE); }
+
+void OpRequest::mark_flag_point(uint8_t flag, string s) {
+#ifdef WITH_LTTNG
+  uint8_t old_flags = hit_flag_points;
+#endif
+  mark_event(s);
+  current = s;
+  hit_flag_points |= flag;
+  latest_flag_point = flag;
+  tracepoint(oprequest, mark_flag_point, reqid.name._type,
+	     reqid.name._num, reqid.tid, reqid.inc, rmw_flags,
+	     flag, s.c_str(), old_flags, hit_flag_points);
+}
