@@ -431,7 +431,7 @@ static void decode_swift_keys(map<string, RGWAccessKey>& m, JSONObj *o)
 {
   RGWAccessKey k;
   k.decode_json(o, true);
-  m[k.id] = k;
+  m[k.subuser] = k;
 }
 
 static void decode_subusers(map<string, RGWSubUser>& m, JSONObj *o)
@@ -528,6 +528,13 @@ void RGWBucketEntryPoint::decode_json(JSONObj *obj) {
   }
 }
 
+void RGWStorageStats::dump(Formatter *f) const
+{
+  encode_json("num_kb", num_kb, f);
+  encode_json("num_kb_rounded", num_kb_rounded, f);
+  encode_json("num_objects", num_objects, f);
+}
+
 void RGWBucketInfo::dump(Formatter *f) const
 {
   encode_json("bucket", bucket, f);
