@@ -151,7 +151,7 @@ TEST(Buffer, constructors) {
 
     unsigned zc_len = 4;
     ::unlink("testfile");
-    ::system("echo ABC > testfile");
+    EXPECT_EQ(0, ::system("echo ABC > testfile"));
     int fd = ::open("testfile", O_RDONLY);
     bufferptr ptr(buffer::create_zero_copy(zc_len, fd, NULL));
     EXPECT_EQ(zc_len, ptr.length());
@@ -179,7 +179,7 @@ protected:
   virtual void SetUp() {
     len = 4;
     ::unlink("testfile");
-    ::system("echo ABC > testfile");
+    EXPECT_EQ(0, ::system("echo ABC > testfile"));
     fd = ::open("testfile", O_RDONLY);
     assert(fd >= 0);
   }
@@ -1076,6 +1076,23 @@ TEST(BufferList, buffers) {
   ASSERT_EQ((unsigned)1, bl.buffers().size());
 }
 
+TEST(BufferList, get_contiguous) {
+  bufferptr a("foobarbaz", 9);
+  bufferptr b("123456789", 9);
+  bufferptr c("ABCDEFGHI", 9);
+  bufferlist bl;
+  bl.append(a);
+  bl.append(b);
+  bl.append(c);
+  ASSERT_EQ(3u, bl.buffers().size());
+  ASSERT_EQ(0, memcmp("bar", bl.get_contiguous(3, 3), 3));
+  ASSERT_EQ(0, memcmp("456", bl.get_contiguous(12, 3), 3));
+  ASSERT_EQ(0, memcmp("ABC", bl.get_contiguous(18, 3), 3));
+  ASSERT_EQ(3u, bl.buffers().size());
+  ASSERT_EQ(0, memcmp("789ABC", bl.get_contiguous(15, 6), 6));
+  ASSERT_LT(bl.buffers().size(), 3u);
+}
+
 TEST(BufferList, swap) {
   bufferlist b1;
   b1.append('A');
@@ -1212,52 +1229,6 @@ TEST(BufferList, is_n_page_sized) {
   }
 }
 
-TEST(BufferList, rebuild_aligned_size_and_memory) {
-  const unsigned SIMD_ALIGN = 32;
-  const unsigned BUFFER_SIZE = 67;
-
-  bufferlist bl;
-  // These two must be concatenated into one memory + size aligned
-  // bufferptr
-  {
-    bufferptr ptr(buffer::create_aligned(2, SIMD_ALIGN));
-    ptr.set_offset(1);
-    ptr.set_length(1);
-    bl.append(ptr);
-  }
-  {
-    bufferptr ptr(buffer::create_aligned(BUFFER_SIZE - 1, SIMD_ALIGN));
-    bl.append(ptr);
-  }
-  // This one must be left alone
-  {
-    bufferptr ptr(buffer::create_aligned(BUFFER_SIZE, SIMD_ALIGN));
-    bl.append(ptr);
-  }
-  // These two must be concatenated into one memory + size aligned
-  // bufferptr
-  {
-    bufferptr ptr(buffer::create_aligned(2, SIMD_ALIGN));
-    ptr.set_offset(1);
-    ptr.set_length(1);
-    bl.append(ptr);
-  }
-  {
-    bufferptr ptr(buffer::create_aligned(BUFFER_SIZE - 1, SIMD_ALIGN));
-    bl.append(ptr);
-  }
-  EXPECT_FALSE(bl.is_aligned(SIMD_ALIGN));
-  EXPECT_FALSE(bl.is_n_align_sized(BUFFER_SIZE));
-  EXPECT_EQ(BUFFER_SIZE * 3, bl.length());
-  EXPECT_FALSE(bl.buffers().front().is_aligned(SIMD_ALIGN));
-  EXPECT_FALSE(bl.buffers().front().is_n_align_sized(BUFFER_SIZE));
-  EXPECT_EQ(5U, bl.buffers().size());
-  bl.rebuild_aligned_size_and_memory(BUFFER_SIZE, SIMD_ALIGN);
-  EXPECT_TRUE(bl.is_aligned(SIMD_ALIGN));
-  EXPECT_TRUE(bl.is_n_align_sized(BUFFER_SIZE));
-  EXPECT_EQ(3U, bl.buffers().size());
-}
-
 TEST(BufferList, is_zero) {
   {
     bufferlist bl;
@@ -1857,9 +1828,9 @@ TEST(BufferList, read_file) {
   bufferlist bl;
   ::unlink("testfile");
   EXPECT_EQ(-ENOENT, bl.read_file("UNLIKELY", &error));
-  ::system("echo ABC > testfile ; chmod 0 testfile");
+  EXPECT_EQ(0, ::system("echo ABC > testfile ; chmod 0 testfile"));
   EXPECT_EQ(-EACCES, bl.read_file("testfile", &error));
-  ::system("chmod +r testfile");
+  EXPECT_EQ(0, ::system("chmod +r testfile"));
   EXPECT_EQ(0, bl.read_file("testfile", &error));
   ::unlink("testfile");
   EXPECT_EQ((unsigned)4, bl.length());
@@ -1870,7 +1841,7 @@ TEST(BufferList, read_file) {
 TEST(BufferList, read_fd) {
   unsigned len = 4;
   ::unlink("testfile");
-  ::system("echo ABC > testfile");
+  EXPECT_EQ(0, ::system("echo ABC > testfile"));
   int fd = -1;
   bufferlist bl;
   EXPECT_EQ(-EBADF, bl.read_fd(fd, len));
